export P=qtwebengine-qt5
export V=5.10.1
export B=1
export MAINTAINER=JuergenFischer

savelog() {
	local f=$1
	rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

savelog package.log
exec > >(tee package.log) 2>&1
date

set -e

if ! [ -d ../qtwebengine ]; then
	cd ..
	git clone --branch v$V git://code.qt.io/qt/qtwebengine.git
	cd qtwebengine
	git submodule update --init
	# git apply ../osgeo4w/qtwebengine.diff
	cd ../osgeo4w
fi

[ -f ninja-win.zip ] || wget https://github.com/ninja-build/ninja/releases/download/v1.8.2/ninja-win.zip
if ! [ -x ./ninja.exe ]; then
	unzip ninja-win.zip ninja.exe
	chmod a+rx ninja.exe
fi

[ -f jom.zip ] || wget http://download.qt.io/official_releases/jom/jom.zip
if ! [ -x ./jom.exe ]; then
	unzip jom.zip jom.exe
	chmod a+rx jom.exe
fi

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(/bin/mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(/bin/mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(/bin/mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(/bin/mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(/bin/cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        /usr/bin/cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        /usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/cygdrive/c/windows32/system32:/cygdrive/c/windows:/bin
        rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in ${ARCHS:-x86 x86_64}; do
	export ARCH

	(
		case "$ARCH" in
		x86)
        		export OSGEO4W_ROOT=/c/OSGeo4W
        		export VCARCH=x86
			export ZLIB=zlib.lib
			;;

		x86_64)
			export OSGEO4W_ROOT=/c/OSGeo4W64
	        	export VCARCH=amd64
			export ZLIB=zdll.lib
			;;

		*)
        		echo invalid arch
        		exit 1
			;;

		esac

		export R=$OSGEO4W_REP/$ARCH/release/qt5/$P
		mkdir -p $R

		for i in libs libs-debug devel; do
			mkdir -p $R/$P-$i
			cp ../qtwebengine/LICENSE.LGPL3 $R/$P-$i/$P-$i-$V-$B.txt
		done

		cat <<EOF >$R/$P-libs/setup.hint
sdesc: "WebEngine for Qt5"
ldesc: "WebEngine for Qt5"
maintainer: JuergenFischer
category: Libs
requires: qt5-libs
EOF

		cat <<EOF >$R/$P-libs-debug/setup.hint
sdesc: "WebEngine for Qt5 (debug libraries)"
ldesc: "WebEngine for Qt5 (debug libraries)"
maintainer: JuergenFischer
category: Libs
requires: $P-libs
external-source: $P-libs
EOF

		cat <<EOF >$R/$P-devel/setup.hint
sdesc: "WebKit for Qt5 (development)"
ldesc: "WebKit for Qt5 (development)"
maintainer: JuergenFischer
category: Libs
requires: $P-libs
external-source: $P-libs
EOF

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat
		fetchenv $OSGEO4W_ROOT/bin/qt5_env.bat

		fetchenv "/c/Program Files (x86)/Microsoft Visual Studio 14.0/VC/vcvarsall.bat" $VCARCH
		export PATH="$PATH:/c/Program Files/CMake/bin:$PWD:$(cygpath -m $PYTHONHOME)"

		cp syncqt.pl $OSGEO4W_ROOT/apps/Qt5/bin/syncqt.pl

		export BUILDDIR=$PWD/build-$ARCH
		mkdir -p $BUILDDIR
		
		cd $BUILDDIR

		qmake CONFIG-=precompile_header ../../qtwebengine/qtwebengine.pro
		[ -f noclean ] || jom clean
		jom

		/usr/bin/tar -C ../.. -cjvf $R/$P-libs/$P-libs-$V-$B-src.tar.bz2 \
			osgeo4w/package.sh \
			osgeo4w/tarinc.pl \
			osgeo4w/syncqt.pl

		(
			echo "/usr/bin/tar -cjvf $R/$P-devel/$P-devel-$V-$B.tar.bz2 \\"
			perl ../tarinc.pl
			/bin/cat <<EOF 
	--xform s,^,apps/qt5/, \\
	lib/cmake \\
	lib/Qt5WebEngine.lib \\
	lib/Qt5WebEngine.prl \\
	lib/Qt5WebEngineCore.lib \\
	lib/Qt5WebEngineCore.prl \\
	lib/Qt5WebEngineWidgets.lib \\
	lib/Qt5WebEngineWidgets.prl \\
	mkspecs \\
	qtwebengine-config.h \\
	qtwebengine-config.pri \\
	qtwebengine-config_p.h

EOF
		) >../tar.sh
			
		/bin/bash ../tar.sh

		/usr/bin/tar -cjvf $R/$P-libs/$P-libs-$V-$B.tar.bz2 \
			--xform s,^,apps/qt5/, \
			bin/Qt5WebEngine.dll \
			bin/Qt5WebEngineCore.dll \
			bin/Qt5WebEngineWidgets.dll \
			bin/QtWebEngineProcess.exe \
			bin/qwebengine_convert_dict.exe \
			plugins/designer/qwebengineview.dll \
			qml/QtWebEngine/plugins.qmltypes \
			qml/QtWebEngine/qtwebengineplugin.dll \
			qml/QtWebEngine/Controls1Delegates/qmldir \
			qml/QtWebEngine/Controls2Delegates/qmldir \
			qml/QtWebEngine/qmldir


		/usr/bin/tar -cjvf $R/$P-libs-debug/$P-libs-debug-$V-$B.tar.bz2 \
			--xform s,^,apps/qt5/, \
			bin/Qt5WebEngineCored.dll \
			bin/Qt5WebEngined.dll \
			bin/Qt5WebEngineWidgetsd.dll \
			bin/QtWebEngineProcess.pdb \
			bin/QtWebEngineProcessd.exe \
			bin/QtWebEngineProcessd.pdb \
			bin/qwebengine_convert_dict.pdb \
			lib/Qt5WebEngineCored.lib \
			lib/Qt5WebEngineCored.prl \
			lib/Qt5WebEngined.lib \
			lib/Qt5WebEngined.prl \
			lib/Qt5WebEngineWidgetsd.lib \
			lib/Qt5WebEngineWidgetsd.prl \
			plugins/designer/qwebengineview.pdb \
			qml/QtWebEngine/qtwebengineplugin.pdb \
			qml/QtWebEngine/qtwebengineplugind.dll \
			qml/QtWebEngine/qtwebengineplugind.pdb
	)
done

date
